/*

Esta funcion realiza las siguientes operaciones:

- Busca proveedores duplicados (por id_arai)
- Si existen duplicados borra el que tenga id persona mayor y remapea los tramites del id persona mayor al menor (ambos con mismo id_arai)
- Guarda logs de conversion en logs_conversion, mapeos en conversion_ids_prov
- En las tablas tmp_ quedan los registros remapeados

*/

create or replace function diaguita.remapeo_proveedores()
returns void as $$
declare
	v_cant numeric;
begin
    	v_cant := 0;

/* 
En estructura.sql

* scp_proveedor_sugerido S
* scp_proveedores_invitados S
* scp_entrega_pliegos S
* scp_oferentes S
** scp_bienes_patrimoniales x2 S
* scp_adjudicaciones S
* scp_ordenes_compras S
* sau_pr_proveedor_rubro
** scp_solicitud_alta_bien_patrimonial_detalles x2 S
** scp_alta_bien_patrimonial_detalles x2 S
* scp_recepcion_ofertas S
* scp_proveedores_sanciones S
* scp_penalidades S
** scp_adjudicaciones_rectificaciones x 2 S
* sau_rt_conceptos_proveedor
** scp_polizas_seguros x2 S
* sau_pr_proveedores
* anx_personas_x_documentos
* anx_personas_domicilios
*/

	/* Agrupa por persona_arai */
	DELETE FROM diaguita.anx_personas_domicilios WHERE persona IN (SELECT MAX(persona) from diaguita.anx_personas WHERE persona_arai IS NOT NULL GROUP BY persona_arai HAVING count(*) > 1);
	DELETE FROM diaguita.anx_personas_x_documentos WHERE persona IN (SELECT MAX(persona) from diaguita.anx_personas WHERE persona_arai IS NOT NULL GROUP BY persona_arai HAVING count(*) > 1);
	DELETE FROM diaguita.sau_rt_conceptos_proveedor WHERE proveedor IN (SELECT MAX(persona) from diaguita.anx_personas WHERE persona_arai IS NOT NULL GROUP BY persona_arai HAVING count(*) > 1);
	DELETE FROM diaguita.sau_pr_proveedor_rubro WHERE persona IN (SELECT MAX(persona) from diaguita.anx_personas WHERE persona_arai IS NOT NULL GROUP BY persona_arai HAVING count(*) > 1);

	CREATE TABLE IF NOT EXISTS diaguita.conversion_ids_prov (
		min integer,
		max integer);

	CREATE TABLE IF NOT EXISTS diaguita.logs_conversion(
		descripcion varchar(255),
		fecha timestamp with time zone DEFAULT now());

	DELETE FROM diaguita.conversion_ids_prov;

	INSERT INTO diaguita.conversion_ids_prov(min,max)
	SELECT MIN(persona) as min, MAX(persona) as max from diaguita.anx_personas WHERE persona_arai IS NOT NULL GROUP BY persona_arai HAVING count(*) > 1;

	/*Remapeos de tablas asociadas al proveedor */

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_proveedores_invitados(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_entrega_pliegos(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_bienes_patrimoniales(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_solicitud_alta_bien_patrimonial_detalles(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_alta_bien_patrimonial_detalles(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_proveedores_sanciones(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_penalidades(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_adjudicaciones_rectificaciones(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_polizas_seguros(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_adjudicaciones(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_oferentes(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_ordenes_compras(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_recepcion_ofertas(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	CREATE TABLE IF NOT EXISTS diaguita.tmp_scp_proveedor_sugerido(
		id integer,
		fecha timestamp with time zone DEFAULT now());

	INSERT INTO diaguita.tmp_scp_proveedores_invitados(id)
	SELECT pi.id_proveedor_invitado as id
	FROM diaguita.scp_proveedores_invitados pi
	INNER JOIN diaguita.conversion_ids_prov as tmp ON pi.persona = tmp.max;

	UPDATE diaguita.scp_proveedores_invitados as pi
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where pi.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_proveedores_invitados:' || v_cant);

	INSERT INTO diaguita.tmp_scp_entrega_pliegos(id)
	SELECT ep.id_entrega_pliego as id
	FROM diaguita.scp_entrega_pliegos ep
	INNER JOIN diaguita.conversion_ids_prov as tmp ON ep.persona = tmp.max;

	UPDATE diaguita.scp_entrega_pliegos as ep
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where ep.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_entrega_pliegos:' || v_cant);

	INSERT INTO diaguita.tmp_scp_bienes_patrimoniales(id)
	SELECT bp.id_bien_patrimonial as id
	FROM diaguita.scp_bienes_patrimoniales bp
	INNER JOIN diaguita.conversion_ids_prov as tmp ON (bp.persona = tmp.max OR bp.titular_bien_tercero=tmp.max);

	UPDATE diaguita.scp_bienes_patrimoniales as bp
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where bp.persona = tmp.max;

	UPDATE diaguita.scp_bienes_patrimoniales as bp
	SET titular_bien_tercero = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where bp.titular_bien_tercero = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_bienes_patrimoniales:' || v_cant);

	INSERT INTO diaguita.tmp_scp_solicitud_alta_bien_patrimonial_detalles(id)
	SELECT bpd.id_solicitud_alta_detalle as id
	FROM diaguita.scp_solicitud_alta_bien_patrimonial_detalles bpd
	INNER JOIN diaguita.conversion_ids_prov as tmp ON (bpd.persona=tmp.max OR bpd.titular_bien_tercero=tmp.max);

	UPDATE diaguita.scp_solicitud_alta_bien_patrimonial_detalles as bpd
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where bpd.persona = tmp.max;

	UPDATE diaguita.scp_solicitud_alta_bien_patrimonial_detalles as bpd
	SET titular_bien_tercero = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where bpd.titular_bien_tercero = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_solicitud_alta_bien_patrimonial_detalles:' || v_cant);

	INSERT INTO diaguita.tmp_scp_alta_bien_patrimonial_detalles(id)
	SELECT abpd.id_alta_detalle as id
	FROM diaguita.scp_alta_bien_patrimonial_detalles abpd
	INNER JOIN diaguita.conversion_ids_prov as tmp ON abpd.persona = tmp.max;

	UPDATE diaguita.scp_alta_bien_patrimonial_detalles as bpd
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where bpd.persona = tmp.max;

	UPDATE diaguita.scp_alta_bien_patrimonial_detalles as bpd
	SET titular_bien_tercero = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where bpd.titular_bien_tercero = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_alta_bien_patrimonial_detalles:' || v_cant);

	INSERT INTO diaguita.tmp_scp_proveedores_sanciones(id)
	SELECT ps.id_proveedor_sancion as id
	FROM diaguita.scp_proveedores_sanciones ps
	INNER JOIN diaguita.conversion_ids_prov as tmp ON ps.persona = tmp.max;

	UPDATE diaguita.scp_proveedores_sanciones as ps
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where ps.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_proveedores_sanciones:' || v_cant);

	INSERT INTO diaguita.tmp_scp_penalidades(id)
	SELECT p.id_penalidad as id
	FROM diaguita.scp_penalidades p
	INNER JOIN diaguita.conversion_ids_prov as tmp ON p.persona = tmp.max;

	UPDATE diaguita.scp_penalidades as pen
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where pen.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_penalidades:' || v_cant);

	INSERT INTO diaguita.tmp_scp_adjudicaciones_rectificaciones(id)
	SELECT rec.id_adjudicacion_rectificacion as id
	FROM diaguita.scp_adjudicaciones_rectificaciones rec
	INNER JOIN diaguita.conversion_ids_prov as tmp ON (rec.persona_nuevo=tmp.max OR rec.persona_original=tmp.max);

	UPDATE diaguita.scp_adjudicaciones_rectificaciones as rec
	SET persona_nuevo = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where rec.persona_nuevo = tmp.max;

	UPDATE diaguita.scp_adjudicaciones_rectificaciones as rec
	SET persona_original = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where rec.persona_original = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_adjudicaciones_rectificaciones:' || v_cant);

	INSERT INTO diaguita.tmp_scp_polizas_seguros(id)
	SELECT seg.id_poliza_seguro as id
	FROM diaguita.scp_polizas_seguros seg
	INNER JOIN diaguita.conversion_ids_prov as tmp ON (seg.productor_seguro=tmp.max OR seg.aseguradora=tmp.max);

	UPDATE diaguita.scp_polizas_seguros as seg
	SET productor_seguro = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where seg.productor_seguro = tmp.max;

	UPDATE diaguita.scp_polizas_seguros as seg
	SET aseguradora = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where seg.aseguradora = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_polizas_seguros:' || v_cant);

	INSERT INTO diaguita.tmp_scp_adjudicaciones(id)
	SELECT a.id_adjudicacion as id
	FROM diaguita.scp_adjudicaciones a
	INNER JOIN diaguita.conversion_ids_prov as tmp ON a.persona = tmp.max;

	UPDATE diaguita.scp_adjudicaciones as adj
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where adj.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_adjudicaciones:' || v_cant);

	INSERT INTO diaguita.tmp_scp_oferentes(id)
	SELECT o.id_oferente as id
	FROM diaguita.scp_oferentes o
	INNER JOIN diaguita.conversion_ids_prov as tmp ON o.persona = tmp.max;

	UPDATE diaguita.scp_oferentes as ofe
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where ofe.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_oferentes:' || v_cant);

	INSERT INTO diaguita.tmp_scp_ordenes_compras(id)
	SELECT oc.id_orden_compra as id
	FROM diaguita.scp_ordenes_compras oc
	INNER JOIN diaguita.conversion_ids_prov as tmp ON oc.persona = tmp.max;

	UPDATE diaguita.scp_ordenes_compras as oc
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where oc.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_ordenes_compras:' || v_cant);

	INSERT INTO diaguita.tmp_scp_recepcion_ofertas(id)
	SELECT ro.id_recepcion_oferta as id
	FROM diaguita.scp_recepcion_ofertas ro
	INNER JOIN diaguita.conversion_ids_prov as tmp ON ro.persona = tmp.max;

	UPDATE diaguita.scp_recepcion_ofertas as ro
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where ro.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_recepcion_ofertas:' || v_cant);

	INSERT INTO diaguita.tmp_scp_proveedor_sugerido(id)
	SELECT ps.id_proveedor_sugerido as id
	FROM diaguita.scp_proveedor_sugerido ps
	INNER JOIN diaguita.conversion_ids_prov as tmp ON ps.persona = tmp.max;

	UPDATE diaguita.scp_proveedor_sugerido as ps
	SET persona = tmp.min
	from (SELECT min,max FROM diaguita.conversion_ids_prov) as tmp
	where ps.persona = tmp.max;

	GET DIAGNOSTICS v_cant = ROW_COUNT;
	INSERT INTO diaguita.logs_conversion(descripcion) VALUES ('scp_proveedor_sugerido:' || v_cant);

	DELETE FROM diaguita.sau_pr_proveedores WHERE persona IN (SELECT MAX(persona) from diaguita.anx_personas WHERE persona_arai IS NOT NULL GROUP BY persona_arai HAVING count(*) > 1);

	DELETE FROM diaguita.anx_personas WHERE persona IN (SELECT MAX(persona) from diaguita.anx_personas WHERE persona_arai IS NOT NULL GROUP BY persona_arai HAVING count(*) > 1);

end;
$$ language plpgsql;



--Remapeo los proveedores
SELECT diaguita.remapeo_proveedores();
  
--Crear tabla y dejar los valores en 
CREATE TEMP TABLE IF NOT EXISTS tmp_arreglo_proveedores AS (
SELECT persona FROM diaguita.anx_personas WHERE persona IN (
    SELECT persona FROM diaguita.anx_personas_x_documentos WHERE numero IN  (
        SELECT pd.numero FROM diaguita.anx_tipos_documento_identidad tdi
        INNER JOIN diaguita.anx_personas_x_documentos pd ON tdi.tipo_de_documento_de_identidad = pd.tipo_de_documento_de_identidad
        INNER JOIN diaguita.anx_personas p ON pd.persona = p.persona
        WHERE (substring(pd.numero,1,2) <> '50' AND substring(pd.numero,1,2) <> '55' AND substring(pd.numero,1,2) <> '51')
        GROUP BY pd.numero HAVING COUNT(*) > 1
    )
)
AND persona_arai isnull
);
                                                                                                                            


--sau_pr_proveedor_rubro
DELETE FROM diaguita.sau_pr_proveedor_rubro where persona IN (SELECT persona FROM tmp_arreglo_proveedores );
--sau_rt_conceptos_proveedor
DELETE FROM diaguita.sau_rt_conceptos_proveedor where proveedor IN (SELECT persona FROM tmp_arreglo_proveedores );
--ssau_pr_proveedores
DELETE FROM diaguita.sau_pr_proveedores where persona IN (SELECT persona FROM tmp_arreglo_proveedores );
--anx_personas_x_documentos
DELETE FROM diaguita.anx_personas_x_documentos where persona IN (SELECT persona FROM tmp_arreglo_proveedores );
--anx_personas_domicilios
DELETE FROM diaguita.anx_personas_domicilios where persona IN (SELECT persona FROM tmp_arreglo_proveedores );
--anx_personas
DELETE FROM diaguita.anx_personas where persona IN (SELECT persona FROM tmp_arreglo_proveedores );                                             


--DOMICILIOS

--Los que no estan
select domicilio_arai FROM diaguita.anx_domicilios GROUP BY domicilio_arai HAVING count(domicilio_arai) > 1;




--los que estan
(SELECT * FROM diaguita.anx_personas_domicilios WHERE domicilio IN (SELECT domicilio FROM diaguita.anx_domicilios WHERE domicilio_arai IN (select domicilio_arai
                    from diaguita.anx_domicilios
                    group by domicilio_arai
                    HAVING count(domicilio_arai) > 1) ORDER BY domicilio));


-- borrado de los duplicados
DELETE FROM diaguita.anx_domicilios WHERE domicilio IN (SELECT domicilio FROM diaguita.anx_domicilios WHERE domicilio_arai IN (select domicilio_arai
                    from diaguita.anx_domicilios
                    group by domicilio_arai
                    HAVING count(domicilio_arai) > 1) AND domicilio NOT IN ((SELECT domicilio FROM diaguita.anx_personas_domicilios WHERE domicilio IN (SELECT domicilio FROM diaguita.anx_domicilios WHERE domicilio_arai IN (select domicilio_arai
                    from diaguita.anx_domicilios
                    group by domicilio_arai
                    HAVING count(domicilio_arai) > 1) ORDER BY domicilio))));



--Borrado final  
DELETE FROM diaguita.anx_domicilios WHERE domicilio IN (SELECT  max(domicilio) as domicilio  FROM diaguita.anx_domicilios  WHERE domicilio_arai notnull GROUP BY domicilio_arai HAVING   Count(*)>1);
